/*
 * Asterisk -- An open source telephony toolkit.
 *
 * Copyright (C) 2007, Saúl Ibarra
 *
 * Saúl Ibarra <saghul@gmail.com>
 *
 * See http://www.asterisk.org for more information about
 * the Asterisk project. Please do not directly contact
 * any of the maintainers of this project for assistance;
 * the project provides a web site, mailing lists and IRC
 * channels for your use.
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License Version 2. See the LICENSE file
 * at the top of the source tree.
 */

/*! \file
 *
 * \brief SpoolCall Application
 *
 * \author Saghul saghul@gmail.com
 *
 * \ingroup applications
 */

/*** MODULEINFO
        <defaultenabled>no</defaultenabled>
 ***/

#include "asterisk.h"

ASTERISK_FILE_VERSION(__FILE__, "$Revision: 0.2 $")

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "asterisk/file.h"
#include "asterisk/logger.h"
#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/module.h"
#include "asterisk/lock.h"
#include "asterisk/app.h"

#define AST_MODULE "SpoolCall"
#define randomize (srand(time(NULL)))
#define random(num) (rand()%(num))
#define RND 10000
#define SPOOLCALL_CFG "spoolcall.conf"

static char qdir[255];

static int max_reintentos;
static int tiempo_reintentos;
static int tiempo_espera;
static int always_delete;
static int archive;

static char *app = "SpoolCall";
static char *synopsis = "Crea un fichero .call en la carpeta spool\n";
static char *descrip = "Permite crear un fichero de llamada en la carpeta de spool con los parametros indicados. \n"
			"Sintaxis: SpoolCall(canal|contexto|extension|[prioridad|maxreintentos|tiemporeintentos|tiempoespera|alwaysdelete|archive]) \n";

static void load_config(void)
{
	struct ast_config   *cfg=NULL;
	struct ast_flags config_flags = {};

	const char *mreintentos;
	const char *treintentos;
	const char *tespera;
	const char *adelete;
	const char *archivestr;
	
	max_reintentos = 2;
	tiempo_reintentos = 60;
	tiempo_espera = 30;
	always_delete = 1;
	archive = 0;

	if(!(cfg = ast_config_load(SPOOLCALL_CFG,config_flags))) {
		ast_log(LOG_NOTICE, "No se ha encontrado el fichero de configuracion spoolcall.conf. Se tomaran los valores por defecto\n");
	}
	else {
		if ((mreintentos = ast_variable_retrieve(cfg, "general", "maxretries"))) {
			if (atoi(mreintentos) >= 0) {
				max_reintentos = atoi(mreintentos);
			}
		}	
		if ((treintentos = ast_variable_retrieve(cfg, "general", "retrytime"))) {
			if (atoi(treintentos) >= 0) {
				tiempo_reintentos = atoi(treintentos);
			}
		}
		if ((tespera = ast_variable_retrieve(cfg, "general", "waittime"))) {
			if (atoi(tespera) >= 0) {
				tiempo_espera = atoi(tespera);
			}
		}
		if ((adelete = ast_variable_retrieve(cfg, "general", "alwaysdelete"))) {
			always_delete = ast_true(adelete);
		}
		if ((archivestr = ast_variable_retrieve(cfg, "general", "archive"))) {
			archive = ast_true(archivestr);
		}
		ast_config_destroy(cfg);
	}
}

static int app_exec(struct ast_channel *chan, void *data)
{
        int res = 0;
        struct ast_flags flags;
        struct ast_module_user *u;
        char *parse;
        AST_DECLARE_APP_ARGS(args,
                AST_APP_ARG(canal);
                AST_APP_ARG(contexto);
                AST_APP_ARG(extension);
                AST_APP_ARG(prioridad);
                AST_APP_ARG(maxreintentos);
                AST_APP_ARG(tiemporeintentos);
                AST_APP_ARG(tiempoespera);
		AST_APP_ARG(adelete);
		AST_APP_ARG(archive);
        );

        if (ast_strlen_zero(data)) {
                ast_log(LOG_WARNING, "%s requiere por lo menos 3 argumentos: canal, contexto y extension\n", app);
                return -1;
        }

        u = ast_module_user_add(chan);

        /* We need to make a copy of the input string if we are going to modify it! */
	parse = ast_strdupa(data);

        AST_STANDARD_APP_ARGS(args, parse);

        if (args.argc < 3) {
                ast_log(LOG_NOTICE, "NO Has puesto los 3 argumentos minimamente necesarios!!\n");
		return -1;
	}
	else {

		load_config();

       		 if (ast_strlen_zero(args.prioridad))
	                args.prioridad = "1";
       		 if (!ast_strlen_zero(args.maxreintentos))
	                max_reintentos = atoi(args.maxreintentos);
       		 if (!ast_strlen_zero(args.tiemporeintentos))
	                tiempo_reintentos = atoi(args.tiemporeintentos);
       		 if (!ast_strlen_zero(args.tiempoespera))
	                tiempo_espera = atoi(args.tiempoespera);
       		 if (!ast_strlen_zero(args.adelete))
	                always_delete = ast_true(args.adelete);
       		 if (!ast_strlen_zero(args.archive))
	                archive = ast_true(args.archive);

	  	 FILE *miFichero;
          	 randomize;
		 /* Directorio de spool */
		 snprintf(qdir, sizeof(qdir), "%s/%s", ast_config_AST_SPOOL_DIR, "outgoing/");
		 int nombre = random(RND);
		 char nomFich[255];
		 sprintf(nomFich,"%s%i.call",qdir,nombre);

			if ((miFichero = fopen(nomFich, "w")) == NULL) {
				ast_log(LOG_WARNING, "Error creando el fichero.\n");
				ast_module_user_remove(u);
				return -1;
			}
			else {
				
				fprintf(miFichero,"Channel: %s\nMaxRetries: %i\nRetryTime: %i\nWaitTime: %i\nContext: %s\nExtension: %s\nPriority: %s\n",args.canal,max_reintentos,tiempo_reintentos,tiempo_espera,args.contexto,args.extension,args.prioridad);
				if (!always_delete) {
					fprintf(miFichero,"AlwaysDelete: No\n");
				}
				if (archive) {
					fprintf(miFichero,"Archive: Yes\n");
				}
				
				fclose(miFichero);
			}



        ast_module_user_remove(u);

        return res;
	}
}

static int unload_module(void)
{
        return ast_unregister_application(app);
}

static int load_module(void)
{
        return ast_register_application(app, app_exec, synopsis, descrip);
}

AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Creador de ficheros .call");

